#include <Stepper.h>

const int stepsPerRevolution = 200;  // Change this to fit the number of steps per revolution for your motor
// Initialize the stepper library on pins 8 through 11:
Stepper myStepper(stepsPerRevolution, 8, 9, 10, 11);

// Step count domains (corresponding to RPM ranges)
const float RPM_domain1_min = 20.0;    // RPM range 40-54
const float RPM_domain1_max = 55.0;
const float RPM_domain2_min = 55.0;    // RPM range 55-79
const float RPM_domain2_max = 65.0;
const float RPM_domain3_min = 65.0;    // RPM range 80-99
const float RPM_domain3_max = 75.0;
const float RPM_domain4_min = 75.0;   // RPM range 100-119
const float RPM_domain4_max = 100.0;

// Tachometer constants
const int hall_pin = 5;               // Hall sensor pin
const float hall_thresh = 8.0;        // Threshold rotations to measure RPM
const unsigned long debounce_delay = 1000; // Debounce delay in microseconds

// Variable to track current position
int currentPosition = 0;
float lastRPM = 0;                     // Variable to store the last RPM value
const float RPM_tolerance = 2.0;       // Tolerance for RPM changes

// Variable to track current gear (domain)
int currentGear = 0;

// Function declaration for debounceHallSensor
bool debounceHallSensor(int pin);

void setup() {
  myStepper.setSpeed(120);             // Set stepper motor speed (adjust as needed)
  
  // Initialize the serial port:
  Serial.begin(9600);
  Serial.println("Setup complete");
  
  // Setup for hall sensor
  pinMode(hall_pin, INPUT);
}

void loop() {
  // Measure RPM
  float rpm_val = measureRPM();

  // Determine target gear based on RPM domain
  int targetGear = 0;

  if (rpm_val >= RPM_domain1_min && rpm_val < RPM_domain1_max) {
    targetGear = 1;
  } else if (rpm_val >= RPM_domain2_min && rpm_val < RPM_domain2_max) {
    targetGear = 2;
  } else if (rpm_val >= RPM_domain3_min && rpm_val < RPM_domain3_max) {
    targetGear = 3;
  } else if (rpm_val >= RPM_domain4_min && rpm_val < RPM_domain4_max) {
    targetGear = 4;
  }

  // Calculate 1-based indices for clarity in comments
  int Matrix_current = currentGear;
  int Matrix_target = targetGear;

  // Define the number of steps needed to move from one gear to another
  int gearSteps[5][5] = {
    // Column index:
    // [1][2][3][4][5]=[Matrix_current] // 5x5 matrix
    {  0,   -6,   -8,  -9,  -11 },  // row index 1 [1]   // From gear 0 (row) to 0, 1, 2, 3, 4 (column)
    {  0,   0,   -2,   -3,  -5 },  // row index 2 [2]   // From gear 1 to 0, 1, 2, 3, 4
    {  0,  2,   0,   -2,   -3 },  // row index 3 [3]   // From gear 2 to 0, 1, 2, 3, 4
    {  0,  3,  2,   0,   -2},  // row index 4 [4]   // From gear 3 to 0, 1, 2, 3, 4
    {  0, 5,  3,  2,   0 }   // row index 5 [5]   // From gear 4 to 0, 1, 2, 3, 4
  };                            // [Matrix_target]

  // Calculate the steps needed to move from current gear to target gear
  int stepsToMove = gearSteps[Matrix_current][Matrix_target] * stepsPerRevolution;

  // Move the stepper motor by the calculated steps

  myStepper.step(stepsToMove);

  // Print movement information
  if (stepsToMove > 0) {
    Serial.print("Row:");
    Serial.print(Matrix_current);
    Serial.print("Collumn:");
    Serial.print(Matrix_target);

    Serial.print("Power decreased-->Moving from gear ");
    Serial.print(currentGear);
    Serial.print(" to ");
    Serial.print(targetGear);
    Serial.print(" with ");
    Serial.print(stepsToMove);
    Serial.println(" steps.");
  } else if (stepsToMove < 0) {
    Serial.print("Power increased-->Moving from gear");
    Serial.print(currentGear);
    Serial.print(" to ");
    Serial.print(targetGear);
    Serial.print(" with ");
    Serial.print(stepsToMove);
    Serial.println(" steps.");
  } else {
    Serial.println("Power input before=-after-->no movement required.");
  }

  // Update the current gear
  currentGear = targetGear;

  // Delay after each movement command
  if (stepsToMove != 0) {
    delay(10000);  // Delay 10 seconds after each movement command
  }

  // Update the lastRPM value
  lastRPM = rpm_val;
}

// Function to measure RPM using hall sensor
float measureRPM() {
  float hall_count = 0.0;
  unsigned long start_time = micros();
  bool last_state = debounceHallSensor(hall_pin);

  // Count rotations until threshold is reached
  while (hall_count < hall_thresh) {
    bool current_state = debounceHallSensor(hall_pin);

    if (last_state == LOW && current_state == HIGH) {
      hall_count++;
    }

    last_state = current_state;
  }

  unsigned long end_time = micros();
  float time_passed = (end_time - start_time) / 1000000.0;
  float rpm_val = (hall_count / time_passed) * 60.0;

  Serial.print("RPM: ");
  Serial.println(rpm_val);
  return rpm_val;
}

// Function to debounce hall sensor input
bool debounceHallSensor(int pin) {
  bool state = digitalRead(pin);
  delayMicroseconds(debounce_delay); // Simple debounce delay
  bool current_state = digitalRead(pin);
  return current_state;
}


